@@ -1,33 +1,42 @@
 import builtins
 import functools
 import importlib
+import inspect
+import math
 import re
 import sys
+from collections.abc import Callable as CCallable
 from collections.abc import Mapping as CMapping
 from collections.abc import MutableMapping as CMutableMapping
 from collections.abc import MutableSequence as CMutableSequence
 from collections.abc import Sequence as CSequence
+from inspect import Parameter
 from types import ModuleType
 from typing import ForwardRef  # type: ignore[import-error]  # pytype (for ForwardRef)
+from typing import _GenericAlias  # type: ignore[attr-defined]
 from typing import (
     TYPE_CHECKING,
     Any,
+    Callable,
     Dict,
+    FrozenSet,
     List,
     Mapping,
     MutableMapping,
     MutableSequence,
     NamedTuple,
+    NewType,
     NoReturn,
     Optional,
     Sequence,
+    Set,
     Tuple,
     Type,
     TypeVar,
     Union,
 )
 from typing import _eval_type as eval_type  # type: ignore[attr-defined]
-from typing import _type_check as type_check  # type: ignore[attr-defined]
+from typing import _type_repr as type_repr  # type: ignore[attr-defined]
 from typing import cast, overload
 
 try:
@@ -81,7 +90,6 @@ if sys.version_info >= (3, 8):
     from typing import get_args, get_origin  # Python 3.8+
 
 elif sys.version_info >= (3, 7):
-    from typing import _GenericAlias  # type: ignore[attr-defined]
 
     def get_origin(tp: object) -> Optional[object]:
         if isinstance(tp, _GenericAlias):  # type: ignore[16]  # pyre
@@ -135,6 +143,115 @@ def _is_typed_dict(tp: object) -> bool:
     return isinstance(tp, _typed_dict_metas)
 
 
+# _is_newtype
+if NewType.__class__.__name__ == "function":  # type: ignore[reportGeneralTypeIssues]  # pyright
+    # Python 3.7 - 3.9
+    def _is_newtype(tp: object) -> bool:
+        return (
+            hasattr(tp, "__class__")
+            and tp.__class__.__name__ == "function"
+            and hasattr(tp, "__qualname__")
+            and tp.__qualname__.startswith("NewType.<locals>")  # type: ignore[attr-defined]
+            and hasattr(tp, "__module__")
+            and tp.__module__ == "typing"
+        )
+
+elif NewType.__class__.__name__ == "type":  # type: ignore[reportGeneralTypeIssues]  # pyright
+    # Python 3.10+
+    def _is_newtype(tp: object) -> bool:
+        return isinstance(tp, NewType)  # type: ignore[arg-type]
+
+else:
+    raise AssertionError(
+        "Do not know how to recognize NewType in this version of Python"
+    )
+
+
+# _inspect_signature
+if sys.version_info >= (3, 10):
+
+    def _inspect_signature(value):
+        return inspect.signature(
+            value,
+            # Don't auto-unwrap decorated functions
+            follow_wrapped=False,
+            # Don't need annotation information
+            eval_str=False,
+        )
+
+else:
+
+    def _inspect_signature(value):
+        return inspect.signature(
+            value,
+            # Don't auto-unwrap decorated functions
+            follow_wrapped=False,
+        )
+
+
+# _type_check
+if sys.version_info >= (3, 11):
+    import types
+    from typing import _GenericAlias  # type: ignore[attr-defined]  # noqa: F811
+    from typing import (  # type: ignore[attr-defined]
+        ClassVar,
+        Final,
+        Generic,
+        ParamSpec,
+        Protocol,
+        _SpecialForm,
+    )
+
+    # Workaround https://github.com/python/cpython/issues/92601
+    # by using Python 3.10's typing._type_check()
+    def _type_check(arg: object, msg: str):
+        """Check that the argument is a type, and return it (internal helper).
+
+        As a special case, accept None and return type(None) instead. Also wrap strings
+        into ForwardRef instances. Consider several corner cases, for example plain
+        special forms like Union are not valid, while Union[int, str] is OK, etc.
+        The msg argument is a human-readable error message, e.g::
+
+            "Union[arg, ...]: arg should be a type."
+
+        We append the repr() of the actual value (truncated to 100 chars).
+        """
+        is_argument = True
+        module = None
+        is_class = False
+
+        invalid_generic_forms = (Generic, Protocol)  # type: tuple[object, ...]
+        if not is_class:
+            invalid_generic_forms += (ClassVar,)
+            if is_argument:
+                invalid_generic_forms += (Final,)
+
+        arg = _type_convert(arg, module=module)
+        if isinstance(arg, _GenericAlias) and arg.__origin__ in invalid_generic_forms:  # type: ignore[reportGeneralTypeIssues]  # pyright
+            raise TypeError(f"{arg} is not valid as type argument")
+        if arg in (Any, NoReturn, Final):
+            return arg
+        if isinstance(arg, _SpecialForm) or arg in (Generic, Protocol):
+            raise TypeError(f"Plain {arg} is not valid as type argument")
+        if isinstance(arg, (type, TypeVar, ForwardRef, types.UnionType, ParamSpec)):
+            return arg
+        if not callable(arg):
+            raise TypeError(f"{msg} Got {arg!r:.100}.")
+        return arg
+
+    # Python 3.10's typing._type_convert()
+    def _type_convert(arg, module=None):
+        """For converting None to type(None), and strings to ForwardRef."""
+        if arg is None:
+            return type(None)
+        if isinstance(arg, str):
+            return ForwardRef(arg, module=module)
+        return arg
+
+else:
+    from typing import _type_check  # type: ignore[attr-defined]
+
+
 __all__ = (
     "trycast",
     "isassignable",
@@ -234,24 +351,35 @@ def trycast(tp, value, failure=None, *, strict=True, eval=True):
         * Literal[...]
         * T extends TypedDict
 
-    Note that unlike isinstance(), this method does NOT consider bool values
-    to be valid int values, as consistent with Python typecheckers:
-        > trycast(int, False) -> None
-        > isinstance(False, int) -> True
+    Similar to isinstance(), this method considers every bool value to
+    also be a valid int value, as consistent with Python typecheckers:
+        > trycast(int, True) -> True
+        > isinstance(True, int) -> True
 
     Note that unlike isinstance(), this method considers every int value to
-    also be a valid float value, as consistent with Python typecheckers:
+    also be a valid float or complex value, as consistent with Python typecheckers:
         > trycast(float, 1) -> 1
+        > trycast(complex, 1) -> 1
         > isinstance(1, float) -> False
+        > isinstance(1, complex) -> False
+
+    Note that unlike isinstance(), this method considers every float value to
+    also be a valid complex value, as consistent with Python typecheckers:
+        > trycast(complex, 1.0) -> 1
+        > isinstance(1.0, complex) -> False
 
     Parameters:
     * strict --
-        If strict=False then trycast will additionally accept
-        mypy_extensions.TypedDict instances and Python 3.8 typing.TypedDict
-        instances for the `tp` parameter. Normally these kinds of types are
-        rejected by trycast with a TypeNotSupportedError because these
-        types do not preserve enough information at runtime to reliably
-        determine which keys are required and which are potentially-missing.
+        * If strict=False then trycast will additionally accept
+          mypy_extensions.TypedDict instances and Python 3.8 typing.TypedDict
+          instances for the `tp` parameter. Normally these kinds of types are
+          rejected by trycast with a TypeNotSupportedError because these
+          types do not preserve enough information at runtime to reliably
+          determine which keys are required and which are potentially-missing.
+        * If strict=False then trycast will treat `NewType("Foo", T)`
+          the same as `T`. Normally NewTypes are rejected by trycast with a
+          TypeNotSupportedError because values of NewTypes at runtime
+          are indistinguishable from their wrapped supertype.
     * eval --
         If eval=False then trycast will not attempt to resolve string
         type references, which requires the use of the eval() function.
@@ -259,8 +387,11 @@ def trycast(tp, value, failure=None, *, strict=True, eval=True):
 
     Raises:
     * TypeNotSupportedError --
-        If strict=True and either mypy_extensions.TypedDict or a
-        Python 3.8 typing.TypedDict is found within the `tp` argument.
+        * If strict=True and either mypy_extensions.TypedDict or a
+          Python 3.8 typing.TypedDict is found within the `tp` argument.
+        * If strict=True and a NewType is found within the `tp` argument.
+        * If a TypeVar is found within the `tp` argument.
+        * If an unrecognized Generic type is found within the `tp` argument.
     * UnresolvedForwardRefError --
         If `tp` is a type form which contains a ForwardRef.
     * UnresolvableTypeError --
@@ -279,7 +410,7 @@ def trycast(tp, value, failure=None, *, strict=True, eval=True):
             )
     else:
         try:
-            tp = type_check(tp, "trycast() requires a type as its first argument.")
+            tp = _type_check(tp, "trycast() requires a type as its first argument.")
         except TypeError:
             if isinstance(tp, tuple) and len(tp) >= 1 and isinstance(tp[0], type):
                 raise TypeError(
@@ -340,26 +471,43 @@ def _trycast_inner(tp, value, failure, options):
     * UnresolvedForwardRefError
     """
     if tp is int:
-        # Do not accept bools as valid int values
-        if isinstance(value, int) and not isinstance(value, bool):
+        # Also accept bools as valid int values
+        if isinstance(value, int):
             return cast(_T, value)
         else:
             return failure
 
     if tp is float:
-        # 1. Accept ints as valid float values
-        # 2. Do not accept bools as valid float values
-        if isinstance(value, float) or (
-            isinstance(value, int) and not isinstance(value, bool)
+        # Also accept ints and bools as valid float values
+        if isinstance(value, float) or isinstance(value, int):
+            return cast(_T, value)
+        else:
+            return failure
+
+    if tp is complex:
+        # Also accept floats, ints, and bools as valid complex values
+        if (
+            isinstance(value, complex)
+            or isinstance(value, float)
+            or isinstance(value, int)
         ):
             return cast(_T, value)
         else:
             return failure
 
     type_origin = get_origin(tp)
+
     if type_origin is list or type_origin is List:  # List, List[T]
         return _trycast_listlike(tp, value, failure, list, options)
 
+    if type_origin is set or type_origin is Set:  # Set, Set[T]
+        return _trycast_listlike(tp, value, failure, set, options)
+
+    if type_origin is frozenset or type_origin is FrozenSet:  # FrozenSet, FrozenSet[T]
+        return _trycast_listlike(
+            tp, value, failure, frozenset, options, covariant_t=True
+        )
+
     if type_origin is tuple or type_origin is Tuple:
         if isinstance(value, tuple):
             type_args = get_args(tp)
@@ -430,6 +578,94 @@ def _trycast_inner(tp, value, failure, options):
                     return value
         return failure
 
+    if type_origin is CCallable:
+        callable_args = get_args(tp)
+        if callable_args == ():
+            # Callable
+            if callable(value):
+                return cast(_T, value)
+            else:
+                return failure
+        else:
+            if len(callable_args) != 2 or (
+                callable_args[0] is not Ellipsis
+                and not isinstance(callable_args[0], list)
+            ):
+                # Python 3.7
+                callable_args = (
+                    list(callable_args[: len(callable_args) - 1]),
+                    callable_args[len(callable_args) - 1],
+                )
+
+            assert len(callable_args) == 2
+            (param_types, return_type) = callable_args
+
+            if return_type is not Any:
+                # Callable[..., T]
+                raise TypeNotSupportedError(
+                    f"trycast cannot reliably determine whether value is "
+                    f"a {type_repr(tp)} because "
+                    f"callables at runtime do not always have a "
+                    f"declared return type. "
+                    f"Consider using trycast(Callable, value) instead."
+                )
+
+            if param_types is Ellipsis:
+                # Callable[..., Any]
+                return _trycast_inner(Callable, value, failure, options)
+
+            assert isinstance(param_types, list)
+            for param_type in param_types:
+                if param_type is not Any:
+                    raise TypeNotSupportedError(
+                        f"trycast cannot reliably determine whether value is "
+                        f"a {type_repr(tp)} because "
+                        f"callables at runtime do not always have "
+                        f"declared parameter types. "
+                        f"Consider using trycast("
+                        f"Callable[{','.join('Any' * len(param_types))}, Any], value) "
+                        f"instead."
+                    )
+
+            # Callable[[Any * N], Any]
+            if callable(value):
+                try:
+                    sig = _inspect_signature(value)
+                except TypeError:
+                    # Not a callable
+                    return failure
+                except ValueError:
+                    # Unable to introspect signature for value.
+                    # It might be a built-in function that lacks signature support.
+                    # Assume conservatively that value does NOT match the requested type.
+                    return failure
+                else:
+                    sig_min_param_count = 0
+                    sig_max_param_count = 0
+                    for expected_param in sig.parameters.values():
+                        if (
+                            expected_param.kind == Parameter.POSITIONAL_ONLY
+                            or expected_param.kind == Parameter.POSITIONAL_OR_KEYWORD
+                        ):
+                            if expected_param.default is Parameter.empty:
+                                sig_min_param_count += 1
+                            sig_max_param_count += 1
+                        elif expected_param.kind == Parameter.VAR_POSITIONAL:
+                            sig_max_param_count = math.inf
+
+                    if sig_min_param_count <= len(param_types) <= sig_max_param_count:
+                        return cast(_T, value)
+                    else:
+                        return failure
+            else:
+                return failure
+
+    if isinstance(tp, _GenericAlias):
+        raise TypeNotSupportedError(
+            f"trycast does not know how to recognize generic type "
+            f"{type_repr(type_origin)}."
+        )
+
     if _is_typed_dict(tp):  # T extends TypedDict
         if isinstance(value, Mapping):
             if options.eval:
@@ -476,6 +712,27 @@ def _trycast_inner(tp, value, failure, options):
         else:
             return failure
 
+    if _is_newtype(tp):
+        if options.strict:
+            supertype_repr = type_repr(tp.__supertype__)  # type: ignore[attribute-error]  # pytype
+            raise TypeNotSupportedError(
+                f"trycast cannot reliably determine whether value is "
+                f"a NewType({tp.__name__!r}, {supertype_repr}) because "
+                f"NewType wrappers are erased at runtime "
+                f"and are indistinguishable from their supertype. "
+                f"Consider using trycast(..., strict=False) to treat "
+                f"NewType({tp.__name__!r}, {supertype_repr}) "
+                f"like {supertype_repr}."
+            )
+        else:
+            supertype = tp.__supertype__  # type: ignore[attribute-error]  # pytype
+            return _trycast_inner(supertype, value, failure, options)
+
+    if isinstance(tp, TypeVar):  # type: ignore[wrong-arg-types]  # pytype
+        raise TypeNotSupportedError(
+            "trycast cannot reliably determine whether value matches a TypeVar."
+        )
+
     if tp is Any:
         if isinstance(tp, type):
             return cast(_T, value)
@@ -696,8 +953,11 @@ def isassignable(value, tp, *, eval=True):
 
     Raises:
     * TypeNotSupportedError --
-        If mypy_extensions.TypedDict or a
-        Python 3.8 typing.TypedDict is found within the `tp` argument.
+        * If strict=True and either mypy_extensions.TypedDict or a
+          Python 3.8 typing.TypedDict is found within the `tp` argument.
+        * If strict=True and a NewType is found within the `tp` argument.
+        * If a TypeVar is found within the `tp` argument.
+        * If an unrecognized Generic type is found within the `tp` argument.
     * UnresolvedForwardRefError --
         If `tp` is a type form which contains a ForwardRef.
     * UnresolvableTypeError --
@@ -813,7 +1073,7 @@ def eval_type_str(tp: str) -> object:
     # 1. Ensure the object is actually a type
     # 2. As a special case, interpret None as type(None)
     try:
-        member = type_check(member, f"Could not resolve type {tp!r}: ")  # type: ignore[16]  # pyre
+        member = _type_check(member, f"Could not resolve type {tp!r}: ")  # type: ignore[16]  # pyre
     except TypeError as e:
         raise UnresolvableTypeError(str(e))
     return member
